﻿Imports System
Imports System.Threading
Imports System.Collections
Imports System.ComponentModel
Imports System.Drawing
Imports System.Windows.Forms
Imports System.IO


Public Class formProceso
    Private numberToCompute As Integer = 0
    Private highestPercentageReached As Integer = 0
    Private iteraciones As Integer


    Private Sub BackgroundWorker1_DoWork(ByVal sender As System.Object,
                     ByVal e As System.ComponentModel.DoWorkEventArgs) Handles hiloSegundoPlano.DoWork
        'Obtener el objeto BackgroundWorker que provocó este evento
        Dim worker As BackgroundWorker = CType(sender, BackgroundWorker)

        ' Assign the result of the computation
        ' to the Result property of the DoWorkEventArgs
        ' object. This is will be available to the 
        ' RunWorkerCompleted eventhandler.
        'Asignar el resultado de la computación a la propiedad Result 
        'del objeto DoWorkEventArgs
        e.Result = ComputeFibonacci(e.Argument, worker, e)
    End Sub

    Function ComputeFibonacci(ByVal n As Integer,
                   ByVal worker As BackgroundWorker,
                   ByVal e As DoWorkEventArgs) As Long

        'No permitimos valores superiores a 91 ni inferiores a 9
        If n < 0 OrElse n > 91 Then
            Throw New ArgumentException( _
                "value must be >= 0 and <= 91", "n")
        End If

        Dim result As Long = 0

        iteraciones = iteraciones + 1

        'Si el usuario ha pulsado el botón cancelar abortar la operación
        If worker.CancellationPending Then
            e.Cancel = True
        Else
            If n < 2 Then
                result = 1
            Else
                result = ComputeFibonacci(n - 1, worker, e) + _
                         ComputeFibonacci(n - 2, worker, e)
            End If

            'Establecemos el porcentaje de progreso realizado del total de la tarea
            Dim percentComplete As Integer = _
                CSng(n) / CSng(numberToCompute) * 100
            If percentComplete > highestPercentageReached Then
                highestPercentageReached = percentComplete
                worker.ReportProgress(percentComplete)
            End If
        End If

        Return result
    End Function

    Private Sub cancelAsyncButton_Click(ByVal sender As System.Object,
                 ByVal e As System.EventArgs) Handles btCancelarProceso.Click
        'Cancelar proceso asíncrono
        hiloSegundoPlano.CancelAsync()

        'Deshabilitar el botón cancelar
        btCancelarProceso.Enabled = False
    End Sub

    Private Sub BackgroundWorker1_ProgressChanged(ByVal sender As Object,
                 ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles hiloSegundoPlano.ProgressChanged
        barraProgreso.Value = e.ProgressPercentage
        lInfo.Text = "Iteraciones Fibonacci: " & CStr(iteraciones)
    End Sub

    Private Sub BackgroundWorker1_RunWorkerCompleted(ByVal sender As Object,
                ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles hiloSegundoPlano.RunWorkerCompleted
        'Manejar el caso en que se produzca un error o excepción
        If (e.Error IsNot Nothing) Then
            MsgBox(e.Error.Message, MsgBoxStyle.OkOnly + MsgBoxStyle.Critical)
        Else
            If e.Cancelled Then
                'Manejar el caso en que el usuario haya cancelado la operación. 
                lInfo.Text = "Proceso cancelado"
            Else
                'Manejar el caso en que la operación haya finalizado con éxito
                lInfo.Text = e.Result.ToString()
            End If
        End If

        'Habilitamos el ListBox del número de iteraciones
        txtNumeroIteraciones.Enabled = True

        'Habilitamos el botón de iniciar proceso
        btIniciarProceso.Enabled = True

        'Deshabilitamos el botón cancelar
        btCancelarProceso.Enabled = False
    End Sub

    Private Sub btIniciarProceso_Click(ByVal sender As System.Object,
                   ByVal e As System.EventArgs) Handles btIniciarProceso.Click
        iteraciones = 0

        lInfo.Text = [String].Empty

        'Deshabilitamos el ListBox con el número de iteraciones
        'hasta que concluya el proceso
        txtNumeroIteraciones.Enabled = False

        'Deshabilitar el botón de inicio de proceso
        'hasta que el proceso asíncrono haya concluido
        btIniciarProceso.Enabled = False

        'Habilitamos el botón cancelar
        'mientras la operación está ejecutándose
        btCancelarProceso.Enabled = True

        'Obtenemos el valor de iteraciones seleccionado por el usuario
        numberToCompute = CInt(txtNumeroIteraciones.Value)

        'Iniciamos el valor de la variable que contendrá el porcentaje del progreso
        highestPercentageReached = 0


        'Iniciamos el proceso asíncrono en segundo plano
        hiloSegundoPlano.RunWorkerAsync(numberToCompute)
    End Sub

    Private Sub btLimpiar_Click(ByVal sender As System.Object,
                 ByVal e As System.EventArgs) Handles btLimpiar.Click
        txtTexto.Text = [String].Empty
    End Sub

    Private Sub btGuardar_Click(ByVal sender As System.Object,
                  ByVal e As System.EventArgs) Handles btGuardar.Click
        dlGuardar.Filter = "Archivos de texto (*.txt)|*.txt|Todos los archivos (*.*)|*.*"
        dlGuardar.FilterIndex = 1
        dlGuardar.DefaultExt = "txt"
        dlGuardar.FileName = "texto"
        dlGuardar.OverwritePrompt = True
        dlGuardar.Title = "Guardar fichero mientras se ejecuta tarea en segundo plano"
        If dlGuardar.ShowDialog Then
            Try
                Dim fichero As New StreamWriter(dlGuardar.FileName)
                fichero.WriteLine(txtTexto.Text)
                fichero.Close()
            Catch ex As Exception
                MsgBox(ex.Message, MsgBoxStyle.OkOnly + _
                       MsgBoxStyle.Exclamation)
            End Try
        End If
    End Sub
End Class
